// Name: 			Lawrence Bush
// RCS Account:		bushl2
// Student Number:	074640477
// Section: 		1
// Email: 			Lawrence_Bush@dps.state.ny.us

// Project 4
// _________________________________________ 
//	NBA.cpp	- 	Main program.
//		  	Reads in player data file.
//		  	Creates draft list of players.
//		  	Implements restricted backtracking algorithm.

#include<iostream>
#include<iomanip>
#include<string>
#include<fstream>
#include<vector>
#include "player.h"
#include "team.h"
#include "draft_list.h"
using namespace std;

int MAX_LEVEL = 12;


// This function performs restricted backtracking on a virtual tree, 
// created from a vector of player objects.
// It recursively calls itself, to explore a tree of players
// for 12 levels.
// 
// Passed in paraveters:
// level - This integer (passed by value) tracks the current level.
// start - This int (passed by value) tracks the number of the first 
//         player to be explored at that level.
// current_team - This is an object of type team that stores the 
//                current_team being explored.
// optimal_team - This is an object of type team that stores the 
//                current optimal team found this far.
// Player_List - This is an object of type draft_list.
//               It contains a vedtor of players to choose the team from.
// * levels, -  This is a pointer to an array of the number of players
//              to explore at each level.
// out - This is an instantiated ostream object which sends output
//       to an open output file.
//       It must be passed in so that the function may use it.

void traverse(int level,int start,team & current_team,team & optimal_team,draft_list Player_List,int * levels,ostream out)
{
	//	cout<<"\nTraverse level "<<level;
	if(level<MAX_LEVEL) {// continue recursive restructed backtracking until we reach level MAX_LEVEL
		//	cout<<"Breadth "<<levels[level]<<"\n";// debug code
		level++;// increment level counter
		int skip = 0;// keeps track of the number of players skipped because they exceeded the salary cap (initially zero)
		
		for(int x=start; ( x<start+levels[level-1]+skip ) && (x<=400) ; ) {
			//	cout<<x;
			if (level == 1) { cout<<((x-skip-start)*100/levels[level-1])<<"% complete\n"; }
			
			
			// Attempt to hire		
			if(   current_team.hire(Player_List[x])   )  { 
				
				// if hire is sucessuful retursively traverse the next player/level
				traverse(level,++x,current_team,optimal_team, Player_List,levels,out);
				if(level >=10) {  // if size constraint is met
					if(	current_team.criteria() ) { // test to see if the team meets constraints
						if( optimal_team < current_team ) {	//	test to see if it is better than the current team
							optimal_team = current_team;	//		store if better
							optimal_team.print(out); // print new team to file
							optimal_team.print(cout);// print new team screen
							cout<<"\ncontinuing...\n";// status of program indicator
							
						} 
					}
				}
				// after recursively traversing the levels, cut the player and 
				// continue for loop trying the next player
				current_team.cut(); 
			} else{++x; ++skip;} // else skip this player and continue for loop
		}
	}
}


// This function is the driver for the restricted backtracking 
// algorithm. 
// It instantiates the current team and optimal team
// and passes them (with other varibles) to the restricted
// backtracking algorithm.
void pick_team(draft_list Player_List,string criteria,int salary_cap,int *levels ,ostream out) {
	// sort by criteria
	
	Player_List.draft_sort(criteria,levels); // call sort on draft list, passing in criteria
	cout<<"\nBacktracking...\n"; // print out program status 
	team optimal_team(salary_cap); // instantiate team to hold best team
	team current_team(salary_cap); // instantiate team to hold current team
	// call recursive restricted backtracking function
	traverse(0,0,current_team,optimal_team, Player_List,levels,out); 
	
	optimal_team.print(out);//print out best team to a file
	optimal_team.print(cout);// print out best team to the screen
}

//
int main(int argc, char *argv[] ) { //  start of main, with input of file argument and the number of arguments
	
	
	if (argc != 17 ) {  //  if there are not 17 arguments
		
		// Example Input
		// input   players.txt pout.txt 30000000 salary 100 10 5 2 2 1 1 2 2 5 10 20
		
		// Error message and command line argument instructions	
		cout<< "Command line arguments:\n"<<
			"The NBA program takes as command line arguments;\n"<<
			"two file names (input file of players and an output file),\n"<< 
			"a salary cap, a sorting criteria and 12 numbers which \n"<<
			"determine the breadth (number of tree nodes) of each \n"<<
			"exploration level.\n"<<
			
			"example:\n"<<
			
			"players.txt out.txt 30000000 value 50 20 10 5 2 1 1 2 5 10 20 50\n";
		exit(0);
	}
	
	int salary_cap = atoi(argv[3]);
	
	string criteria = argv[4];
	
	
	
	//open out file
	ofstream out;
	out.open(argv[2],ios::out);
	if (out == NULL) {// error of out file could not be opened
		cerr << "Error, could not open the file file" << endl;
		//	cin >>i;
		exit(0);
	}
	// read in salary cap and criteria
	cout << "Salary Cap = "<<salary_cap<<"\n";
	out << "Salary Cap = "<<salary_cap<<"\n";
	out<<"Criteria : " << criteria<<"\n";
	cout<<"Criteria : " << criteria<<"\n";
	
	int levels[12];
	// read in and print out levels
	for(int l=0;l<12;l++) { levels[l]= atoi(argv[l+5]);}
	cout<<"Players per level: ";
	out<<"Players per level: ";
	for(l=0;l<12;l++) {
		cout<<levels[l]<<" ";// print scope to screen
		out<<levels[l]<<" ";// print scope to file
	}
	//cin>>l;
	
	ifstream in;
	in.open(argv[1],ios::in);
	if (in == NULL) {// error message if in file could not be opened
		cerr << "Error, could not open the file myfile" << endl;
		//	cin >>i;
		exit(0);
	}
	// Instantiate variables for player file input
	string name;
	char position;
	int FT;
	int IN;
	int WT;
	float PTS;
	float REB;
	float AST;
	int REQ_SALARY;
	float RATING;
	cout<<"files opened"<<endl;
	
	//This is an example of one line from a player input file.
	//Abdul-Wahad,Tariq; G 6 6 223 11.4 4.8 1.6 1594920 12.42
	
	// create vector to store player list	
	draft_list list_of_players;
	
	int countlines=0;
	// read file and push players into player list
	while(in>>name>>position>> FT>>IN>>WT>>PTS>>REB>>AST>>REQ_SALARY>>RATING){
		list_of_players.insert(player(name,position, FT,IN,WT,PTS,REB,AST,REQ_SALARY,RATING) );
		countlines++;
	}
	// Output number of players
	cout<<"\n "<<countlines<< "Players read in from file\n";
	//***
	
	in.close();//close input file
	// program status output
	cout<<"Parsing done\n picking team..."<<endl;
	// Pass list of players, criteria, salary cap, levels array and iostream out object
	// to the pick_team function.
	pick_team( list_of_players,criteria,salary_cap ,levels,out);
	
	
	return 0;
	
	
}