//team.cpp -	Contains functions for the team class.

// Name: 			Lawrence Bush
// RCS Account:		bushl2
// Student Number:	074640477
// Section: 		1
// Email: 			Lawrence_Bush@dps.state.ny.us

// Project 4
// _________________________________________ 
#include <iostream>
#include <iomanip>

#include "team.h"
using namespace std;

const std::string NULL_STRING = "";
// default constructor 
// initializes the data members to 0
// unless otherwise specified i.e. salary_cap
team::team(int salary_cap_in) : salary_cap(salary_cap_in),guards(0),centers(0)
,forwards(0),total_salary(0),team_rating(0),team_size(0) {}   
// copy constructor
team::team(const team & t) {// pass in team to copy
	// copy salary cap
	salary_cap = t.salary_cap;
	// hire all players from copied team
	for(int i=0;i<t.size();i++){
		hire(t[i]);
	}
	// copy remaining data members
	name = t.name;
	guards = t.guards; 
	centers = t.centers;
	forwards = t.forwards;
	total_salary = t.total_salary;
	salary_cap = t.salary_cap;
	team_rating = t.team_rating;	
	team_size = t.team_size;
}
// print function to print whole team
void team::print(ostream out) {
	// Print Header for team table
	out << "\nPrint Team \nName              Pos. Height  Weight Points Rebounds Assists  Salary Rating\n";
	// Iterate through team vector, 
	// dereferencing and printing each player,
	// using the ostream object out, which is passed in 
	// and the overloaded << operator of player.
	for( vector<player>::iterator i = roster.begin() ; i < roster.end() ; i++ ) {
		out << *i;
	}
	// print out team rating and salary
	out <<"Team Rating :  " << team_rating;
	out <<"\nTeam Salary :  " << total_salary;
}
// This function that tests if the team meets specified criteria
bool team::criteria(){
	// test to see if the team has between 10 and 12 players
	if(team_size<10 || team_size>12) return 0;
	// test to see if the team has 4 guards, 3 forwards and 1 center
	if(guards<4 || forwards<3 || centers < 1) return 0;
	// test to see if the team salary is below the salary cap
	if (total_salary>salary_cap) return 0;
	
	return 1; // If it passes all the tests, ruturn 1 for successful
}
// remove a player object from the container
team::cut(){
	// Create temporary player object 
	// taken from the back of the list.
	// This is the one that will be removed.
	player trade = roster.back();
	// Decrease team salary by the traded player's salary
	total_salary -= trade.get_salary();
	// Decrease team_size
	team_size--;
	// Decrease team rating by the traded player's rating
	team_rating -= trade.get_rating();
	
	// Test to see what position the player plays
	// and decrement the appropriate counter.
	if(trade.get_position() == 'G' ) guards--;
	if(trade.get_position() == 'C' ) centers--;
	if(trade.get_position() == 'F' ) forwards--;
	
	// Remove player from roster
	roster.pop_back();
}
// (hire) insert a player object into the container
bool team::hire(player trade){
	// Reset team rating to minimize float rounding error
	if (team_size == 0) {team_rating=0;}
	// Hire if the team salary would still be 
	// under the cap after the proposed player is hired.
	if ( (total_salary + trade.get_salary()) <= salary_cap ) {
		total_salary += trade.get_salary(); // add in new salary 
		team_size++; // increment team
		team_rating += trade.get_rating();
		// Test to see what position the player plays
		// and increment the appropriate counter.
		if(trade.get_position() == 'G' ) guards++;
		if(trade.get_position() == 'C' ) centers++;
		if(trade.get_position() == 'F' ) forwards ++;
		
		// Add player to roster
		roster.push_back(trade);
		return 1;// Return bool for true if hire is successful
	}
	else {
		// don't hire, return 0
		return 0; }
}

//*********************
// Overloaded Operators
//*********************
// assessor operator, returns player k
player team::operator[](int k) const
{
	return roster[k];
}
// binary comparitor of team.rating
bool team::operator<( const team & rhs) const {
	// returns 1 if left team rating < rhs team rating
	// or 0 otherwise
	return team_rating < rhs.team_rating;
} 
// assignment operator
const team & team::operator=(const team & rhs) {
	if (this != &rhs) // identity check (no self assignment)
	{	
		// copy all data members
		roster = rhs.roster; // uses vector assignment operator
		name = rhs.name;
		guards = rhs.guards; // min 4
		centers = rhs.centers; // min 1
		forwards = rhs.forwards; // min 3
		total_salary = rhs.total_salary;
		salary_cap = rhs.salary_cap;
		team_rating = rhs.team_rating;
		team_size = rhs.team_size;
		
	}
	return *this; // allows for cascading assignment
} // end assignment operator